/*
 *
 *
 *    device.cc
 *
 *    C++ source code for class my::iocore member functions and/or static
 *    member data.
 *
 *    =========================================================================
 *
 *    Copyright (C) 2003 by Matt Roberts, KK5JY.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    =========================================================================
 *
 *
 */


#include <sys/types.h>
#include <unistd.h>
#include <errno.h>

#include <my++/device.h>


/*
 *
 *   I/O routines
 *
 */

size_t my::idevice::get (void *buf, size_t len) {
	if (getfd() < 0)
		throw device::exception(this, errno);

	// read from the device
	ssize_t count = read_vp ((void*)buf, len);

	if (count < 0)
		throw device::exception(this, errno);
	return count;
}

size_t my::idevice::get (char *buf, size_t len) {
	if (getfd() < 0)
		throw device::exception(this, errno);

	// read from the device
	ssize_t count = read_vp ((void*)(buf), len);

	if (count < 0)
		throw device::exception(this, errno);
	return count;
}

size_t my::idevice::get (unsigned char *buf, size_t len) {
	if (getfd() < 0)
		throw device::exception(this, errno);

	// read from the device
	ssize_t count = read_vp ((void*)(buf), len);

	if (count < 0)
		throw device::exception(this, errno);
	return count;
}

size_t my::odevice::put (const void *buf, size_t len) {
	if (getfd() < 0)
		throw device::exception(this, errno);

	// write to the device
	ssize_t count = write_vp ((void*)buf, len);

	if (count < 0)
		throw device::exception(this, errno);
	return count;
}

size_t my::odevice::put (const char *buf, size_t len) {
	if (getfd() < 0)
		throw device::exception(this, errno);

	// write to the device
	ssize_t count = write_vp ((void*)(buf), len);

	if (count < 0)
		throw device::exception(this, errno);
	return count;
}

size_t my::odevice::put (const unsigned char *buf, size_t len) {
	if (getfd() < 0)
		throw device::exception(this, errno);

	// write to the device
	ssize_t count = write_vp ((void*)(buf), len);

	if (count < 0)
		throw device::exception(this, errno);
	return count;
}

/* EOF */
